using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using WiimoteLib;
using System;

static class Program
{ static void Main() { Game1 game = new Game1(); game.Run(); }
  public class Game1 : Microsoft.Xna.Framework.Game
  { private GraphicsDeviceManager g;
    private Model model;
    private BasicEffect effect;
    private float positionX = 0.0f, positionY = 0.0f, positionZ = 0.0f;
    private float scaleX = 1.0f, scaleY = 1.0f, scaleZ = 1.0f;
    private float rotationX = 0.0f, rotationY = 0.0f, rotationZ = 0.0f;
    private Form1   form    = new Form1();
    private Wiimote wiimote;
    public Game1() 
    { g = new GraphicsDeviceManager( this ); 
      this.Exiting += new System.EventHandler( OnExit );
      //Connect a common event handler to all 5 RadioButtons
      for ( int i = 0; i < Form1.nRadioButtons; i++)
        form.radiobutton[i].Click += new EventHandler( OnRadioButtonClick );
    }

    protected override void Initialize()
    { g.PreferredBackBufferWidth = 600;
      g.PreferredBackBufferHeight = 600;
      g.ApplyChanges();
      g.IsFullScreen = false;
      Window.AllowUserResizing = true;
      Window.Title = "XNA Using Wiimote";
      try { wiimote = new Wiimote();
            wiimote.Connect();
            wiimote.SetReportType( InputReport.IRAccel, true );
            wiimote.SetLEDs( false, true, true, false );
          }
      catch { System.Windows.Forms.MessageBox.Show( "Can't find a Wiimote" ); Exit(); }
      form.Location = new System.Drawing.Point( Window.ClientBounds.Right + 8, 
                                                Window.ClientBounds.Top );
      form.Size = new System.Drawing.Size( 100, Window.ClientBounds.Height );
      form.Show();
      base.Initialize();
    }

    protected override void LoadContent()
    { model = Content.Load< Model >( "Content\\tiger" );
      effect = (BasicEffect)model.Meshes[0].Effects[0];
      effect.View = Matrix.CreateLookAt( new Vector3( 0.0f, 0.0f, 4.0f ), 
                                         Vector3.Zero, Vector3.Up);
      effect.Projection =
        Matrix.CreatePerspectiveFieldOfView( MathHelper.Pi / 4, 1f, 0.1f, 1000.0f );
    }

    protected override void Update(GameTime gameTime)
    { WiimoteState s = wiimote.WiimoteState;
      for ( int i=0; i < form.checkbox.Length; i++ ) form.checkbox[i].Checked=false;
      if (s.ButtonState.Left)  { form.checkbox[ 0].Checked=true; positionX -= 0.01f; } 
      if (s.ButtonState.Right) { form.checkbox[ 1].Checked=true; positionX += 0.01f; } 
      if (s.ButtonState.Up)    { form.checkbox[ 2].Checked=true; positionY += 0.01f; } 
      if (s.ButtonState.Down)  { form.checkbox[ 3].Checked=true; positionY -= 0.01f; } 
      if (s.ButtonState.A)     { form.checkbox[ 4].Checked=true; scaleX += 0.01f; }
      if (s.ButtonState.B)     { form.checkbox[ 5].Checked=true; scaleX -= 0.01f; }
      if (s.ButtonState.Minus) { form.checkbox[ 6].Checked=true; scaleX=scaleY=scaleZ-=0.01f; } 
      if (s.ButtonState.Plus)  { form.checkbox[ 7].Checked=true; scaleX=scaleY=scaleZ+=0.01f; } 
      if (s.ButtonState.Home)  { form.checkbox[ 8].Checked=true; scaleX=scaleY=scaleZ=1f; 
                                                                 positionX=positionY=positionZ=0f; 
                                                                 rotationX=rotationY=rotationZ=0f; } 
      if (s.ButtonState.One)   { form.checkbox[ 9].Checked=true; rotationX += 0.01f; }
      if (s.ButtonState.Two)   { form.checkbox[10].Checked=true; rotationZ += 0.01f; }
      Point3F p = s.AccelState.Values; //read from the 3 acceleration sensors
      try { form.trackbar[0].Value = (int)(p.X * 10); } catch {}
      try { form.trackbar[1].Value = (int)(p.Y * 10); } catch {}
      try { form.trackbar[2].Value = (int)(p.Y * 10); } catch {}
      //p.X, p.Y can be negative, p.Z will be ignored
      float px = p.X > 0 ? p.X : - p.X; //px is always positive
      float py = p.Y > 0 ? p.Y : - p.Y; //py is always positive
      //ignore the lower absolute value to avoid confusing rotations
           if ( px >= py ) rotationX += 0.01f * p.X; //
      else if ( py >  px ) rotationY += 0.01f * p.Y;
    }

    protected override void Draw( GameTime gameTime )
    { g.GraphicsDevice.Clear( Color.DarkBlue );
      effect.EnableDefaultLighting();
      effect.World = Matrix.CreateScale( scaleX, scaleY, scaleZ ) *
                     Matrix.CreateRotationX( rotationX ) *
                     Matrix.CreateRotationY( rotationY ) *
                     Matrix.CreateRotationZ( rotationZ ) *
                     Matrix.CreateTranslation( positionX, positionY, positionZ );
      model.Meshes[0].Draw();
    }

    private void OnExit( object sender, EventArgs e )
    { wiimote.SetLEDs(false, false, false, false);
      wiimote.Disconnect();
      form.Close();
    }

    //common event handler for all 5 RadioButtons
    public void OnRadioButtonClick( Object sender, EventArgs e )
    { wiimote.SetLEDs  ( form.radiobutton[0].Checked, form.radiobutton[1].Checked, 
                         form.radiobutton[2].Checked, form.radiobutton[3].Checked);
      wiimote.SetRumble( form.radiobutton[4].Checked );
    }
  } // end of class Game1
} // end of class Program